#     Copyright (C) 2007 William McCune
#
#     This file is part of the LADR Deduction Library.
#
#     The LADR Deduction Library is free software; you can redistribute it
#     and/or modify it under the terms of the GNU General Public License
#     as published by the Free Software Foundation; either version 2 of the
#     License, or (at your option) any later version.
#
#     The LADR Deduction Library is distributed in the hope that it will be
#     useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with the LADR Deduction Library; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
#

# system imports

import os, wx, re, copy
import time, _thread, tempfile, subprocess, signal

# local imports

import utilities
from files import *
from platforms import *
from wx_utilities import *
from my_setup import *
from options import *

def run_and_wait(command, input = '', fin = None):

    if not fin:
        fin  = tempfile.TemporaryFile('w+b')  # stdin
        fin.write(input)
        fin.seek(0)

    fout = tempfile.TemporaryFile('w+b')  # stdout
    ferr = tempfile.TemporaryFile('w+b')  # stderr

    if Win32():
        # creationflag says not to pop a DOS box
        process = subprocess.Popen(command,stdin=fin,stdout=fout,stderr=ferr,
                                  creationflags=win32process.CREATE_NO_WINDOW)
    else:
        process = subprocess.Popen(command,stdin=fin,stdout=fout,stderr=ferr)

    exit_code = process.wait()
    fout.seek(0)
    output = fout.read()
    ferr.seek(0)
    error = ferr.read()
    fin.close()
    fout.close()
    ferr.close()
    return (exit_code, output, error)

def isofilter_command(program_name):
    fullpath = os.path.join(bin_dir(), program_name)
    if not binary_ok(fullpath):
        return None
    else:
        return [fullpath]

def ops_in_interp(s):
    i = s.find('interpretation(')
    ops = []
    if i >= 0:
        j = s.find(').', i+1)
        interp = s[i:j+2]
        r = re.compile('(?:function|relation)\(([^,(]*)')
        m = r.search(interp)
        while m:
            op = m.group(1)
            if op != '=':
                ops.append(op)
            m = r.search(interp, m.end())
    return ops
    
class Prover9:

    name = 'Prover9'
    solution_name = 'Proof'
    box_name = 'Proof Search'
    solution_ext = 'proof'
    some_message = 'Some, but not all, of the requested proofs were found.'

    logo_path = os.path.join(image_dir(), 'prover9-5a-128t.gif')

    # Compile regular expression for extracting stats from stderr.

    r_info = re.compile('Given=(\d+)\. Generated=(\d+)\. Kept=(\d+)\. '
                        'proofs=(\d+)\.User_CPU=(\d*\.\d*),')

    exits = {}
    exits[0]   = 'Proof'
    exits[1]   = 'Fatal Error'
    exits[2]   = 'Exhausted'
    exits[3]   = 'Memory Limit'
    exits[4]   = 'Time Limit'
    exits[5]   = 'Given Limit'
    exits[6]   = 'Kept Limit'
    exits[7]   = 'Action Exit'
    exits[101] = 'Interrupted'
    exits[102] = 'Crashed'
    exits[-9]  = 'Killed' # Linux, Mac
    exits[-1]  = 'Killed' # Win32

    def search_command(self):
        fullpath = os.path.join(bin_dir(), 'prover9')
        if not binary_ok(fullpath):
            return None
        else:
            return [fullpath]

    def success_command(self):
        fullpath = os.path.join(bin_dir(), 'prooftrans')
        if not binary_ok(fullpath):
            return None
        else:
            return [fullpath]

    def exists_solution(self, exit_code, output):
        return output.find('== PROOF ==') >= 0

    def count_solutions(self, solutions):
        return solutions.count('== PROOF ==')

    def exit_message(self, code):
        if code in list(self.exits.keys()):
            return self.exits[code]
        else:
            return 'unknown exit code: %d' % code

    def logo_bitmap(self):
        if not os.access(self.logo_path, os.R_OK):
            error_dialog('The logo file %s cannot be found.' % self.logo_path)
            return None
        else:
            return wx.Image(self.logo_path,
                            wx.BITMAP_TYPE_GIF).ConvertToBitmap()

    def get_info_from_stderr(self, lines):
        stats = utilities.grep_last('Given', lines)
        time  = utilities.grep_last('User_CPU', lines)
        if stats and time:
            line = stats.strip() + time.strip()
            m = self.r_info.match(line)
            if m:
                return [('CPU Seconds', m.groups()[4]),
                        ('Given',       m.groups()[0]),
                        ('Generated',   m.groups()[1]),
                        ('Kept',        m.groups()[2]),
                        ('Proofs',      m.groups()[3])]
            else:
                error_dialog('get_stderr_info, failed to match')

        return [('CPU Seconds', '?'),
                ('Given',       '?'),
                ('Generated',   '?'),
                ('Kept',        '?'),
                ('Proofs',      '?')]

    def reformatter(self, parent, proofs, saved_solution):
        n = self.count_solutions(proofs)
        return Reformat_proof(parent, proofs, n, saved_solution)
        
# end class Prover9

class Mace4:

    name = 'Mace4'
    solution_name = 'Model'
    box_name = 'Model/Counterexample Search'
    solution_ext = 'model'
    some_message = ''

    logo_path = os.path.join(image_dir(), 'mace4-90t.gif')
    
    # Compile regular expression for extracting stats from stderr.
    # Domain_size=8. Models=0. User_CPU=8.00.
    r_info = re.compile('Domain_size=(\d+)\. Models=(\d+)\. User_CPU=(\d*\.\d*)\.')

    exits = {}
    exits[0]   = 'Model(s)'
    exits[1]   = 'Fatal Error'
    exits[2]   = 'Exhausted (no)'
    exits[3]   = 'Exhausted (yes)'
    exits[4]   = 'Time Limit (yes)'
    exits[5]   = 'Time Limit (no)'
    exits[6]   = 'Mem Limit (yes)'
    exits[7]   = 'Mem Limit (no)'
    exits[101] = 'Interrupted'
    exits[102] = 'Crashed'
    exits[-9]  = 'Killed' # Linux, Mac
    exits[-1]  = 'Killed' # Win32

    def search_command(self):
        fullpath = os.path.join(bin_dir(), 'mace4')
        if not binary_ok(fullpath):
            return None
        else:
            return [fullpath, '-c']

    def success_command(self):
        fullpath = os.path.join(bin_dir(), 'interpformat')
        if not binary_ok(fullpath):
            return None
        else:
            return [fullpath]

    def exists_solution(self, exit_code, output):
        return output.find('== MODEL ==') >= 0

    def count_solutions(self, solutions):
        return solutions.count('interpretation')

    def exit_message(self, code):
        if code in list(self.exits.keys()):
            return self.exits[code]
        else:
            return 'unknown exit code: %d' % code

    def logo_bitmap(self):
        if not os.access(self.logo_path, os.R_OK):
            error_dialog('The logo file %s cannot be found.' % self.logo_path)
            return None
        else:
            return wx.Image(self.logo_path,
                            wx.BITMAP_TYPE_GIF).ConvertToBitmap()

    def get_info_from_stderr(self, lines):
        line = utilities.grep_last('Domain_size=', lines)
        if line:
            m = self.r_info.match(line)
            if m:
                return [('CPU Seconds', m.groups()[2]),
                        ('Domain Size', m.groups()[0]),
                        ('Models'     , m.groups()[1])]
            else:
                error_dialog('get_stderr_info, failed to match')
        return [('CPU Seconds', '?'),
                ('Domain Size', '?'),
                ('Models'     , '?')]

    def reformatter(self, parent, models, saved_solution):
        n = self.count_solutions(models)
        return Reformat_model(parent, models, n, saved_solution)

# end class Mace4

class Reformat_proof:
    def __init__(self, parent, proofs, num_proofs, saved_flag):

        self.parent = parent
        self.proofs = proofs
        self.num_proofs = num_proofs
        self.saved_flag = saved_flag

        self.choices = ['standard', 'parents_only', 'xml', 'ivy', 'hints']
        self.choice = self.choices[0]

        self.dlg = dlg = wx.Dialog(parent, -1, title="Reformat Proof(s)",
                                   pos=pos_for_center((0,0)))

        rb = wx.RadioBox(dlg, -1, 'Format', choices=self.choices,
                         majorDimension=1, style=wx.RA_SPECIFY_COLS)
        dlg.Bind(wx.EVT_RADIOBOX, self.on_rb, rb)

        options_box = wx.StaticBox(dlg, -1, 'Options')
        box_sizer = wx.StaticBoxSizer(options_box, wx.VERTICAL)
        self.expand_cb = wx.CheckBox(dlg, -1, 'Expand Proof')
        self.renumber_cb = wx.CheckBox(dlg, -1, 'Renumber Proof')
        self.striplabels_cb = wx.CheckBox(dlg, -1, 'Remove Labels')
        self.hl_cb = wx.CheckBox(dlg, -1, 'Hints Label')
        self.hl_ctrl = wx.TextCtrl(dlg, -1, '', size=(100, -1))

        box_sizer.Add(self.expand_cb)
        box_sizer.Add(self.renumber_cb)
        box_sizer.Add(self.striplabels_cb)
        subsizer = wx.BoxSizer(wx.HORIZONTAL)
        subsizer.Add(self.hl_cb, 0, wx.CENTER, 0)
        subsizer.Add(self.hl_ctrl, 0, wx.CENTER, 0)
        box_sizer.Add(subsizer)

        cancel_btn = wx.Button(dlg, -1, 'Cancel')
        dlg.Bind(wx.EVT_BUTTON, self.on_cancel, cancel_btn)
        ok_btn = wx.Button(dlg, -1, 'OK')
        dlg.Bind(wx.EVT_BUTTON, self.on_ok, ok_btn)

        sizer1 = wx.BoxSizer(wx.HORIZONTAL)
        sizer1.Add(rb, 0, wx.ALL|wx.CENTER, 10)
        sizer1.Add(box_sizer, 0, wx.ALL|wx.CENTER, 10)

        sizer2 = wx.BoxSizer(wx.HORIZONTAL)
        sizer2.Add(cancel_btn, 0, wx.ALL, 10)
        sizer2.Add(ok_btn, 0, wx.ALL, 10)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(sizer1, 0, wx.CENTER, 10)
        sizer.Add(sizer2, 0, wx.CENTER, 10)
        dlg.SetSizer(sizer)
        sizer.Fit(dlg)

        self.grayout_options()
        if Win32():
            rc = dlg.Show()  # if modal, I can't get the new fame to the top!
        else:
            rc = dlg.ShowModal()

    def grayout_options(self):
        if self.choice in ['standard', 'parents_only', 'xml']:
            self.expand_cb.Enable(True)
            self.renumber_cb.Enable(True)
            self.striplabels_cb.Enable(True)
            self.hl_cb.Enable(False)
            self.hl_ctrl.Enable(False)
        elif self.choice == 'ivy':
            self.expand_cb.Enable(False)
            self.renumber_cb.Enable(True)
            self.striplabels_cb.Enable(False)
            self.hl_cb.Enable(False)
            self.hl_ctrl.Enable(False)
        elif self.choice == 'hints':
            self.expand_cb.Enable(True)
            self.renumber_cb.Enable(False)
            self.striplabels_cb.Enable(True)
            self.hl_cb.Enable(True)
            self.hl_ctrl.Enable(True)
        else:
            error_dialog('grayout_options: unknown proof format')

    def command(self):
        command = [os.path.join(bin_dir(), 'prooftrans'), self.choice]
        if self.expand_cb.IsEnabled() and self.expand_cb.IsChecked():
            command.append('expand')
        if self.renumber_cb.IsEnabled() and self.renumber_cb.IsChecked():
            command.append('renumber')
        if self.striplabels_cb.IsEnabled() and self.striplabels_cb.IsChecked():
            command.append('striplabels')
        if self.hl_cb.IsEnabled() and self.hl_cb.IsChecked():
            command.append('-label')
            lab = self.hl_ctrl.GetValue().strip()
            if lab != '':
                command.append(lab)
        return command
        
    def on_rb(self, evt):
        self.choice = self.choices[evt.GetInt()]
        self.grayout_options()

    def on_ok(self, evt):
        command = self.command()
        (exit_code, output, err) = run_and_wait(command, input=self.proofs)
        self.dlg.Destroy()
        if exit_code != 0:
            error_dialog("Error reformatting proofs")
        else:
            args = ' '.join(command[1:])
            n = self.num_proofs
            if self.num_proofs == 1:
                title = 'Reformatted Proof (%s)' % (args)
            else:
                title = 'Reformatted Proofs (%s, %d proofs)' % (args, n)
            
            frame = Text_frame(self.parent, to_top(self.parent).box_font,
                               title,
                               output,
                               extension = 'proof',
                               saveas=True,
                               saved_flag = self.saved_flag,
                               off_center=20)
            frame.Show(True)
            frame.Raise()

    def on_cancel(self, evt):
        self.dlg.Destroy()

# class Reformat_proof

class Reformat_model:
    def __init__(self, parent, models, num_models, saved_flag):
        self.parent = parent
        self.models = models
        self.saved_flag = saved_flag
        self.num_models = num_models

        self.choices = ['standard', 'standard2', 'portable', 'tabular',
                        'raw', 'cooked', 'tex', 'xml']
        menu = wx.Menu()
        self.map = {}
        for item in self.choices:
            id = wx.NewId()
            self.map[id] = item
            menu.Append(id, item)
            self.parent.Bind(wx.EVT_MENU, self.on_select, id=id)
        self.parent.PopupMenu(menu)
        menu.Destroy()

    def on_select(self, evt):
        item = self.map[evt.GetId()]
        command = [os.path.join(bin_dir(), 'interpformat'), item]
        (exit_code, output, err) = run_and_wait(command, input=self.models)
        if exit_code != 0:
            error_dialog("Error reformatting models")
        else:
            args = ' '.join(command[1:])
            n = self.num_models
            if self.num_models == 1:
                title = 'Reformatted Model (%s)' % (args)
            else:
                title = 'Reformatted Models (%s, %d models)' % (args, n)
            
            frame = Text_frame(self.parent, to_top(self.parent).box_font,
                               title,
                               output,
                               extension = 'model',
                               saveas=True,
                               saved_flag = self.saved_flag,
                               off_center=40)
            frame.Show(True)

# class Reformat_model

class Run_program:
    def __init__(self, parent, program, input):
        self.parent = parent
        self.program = program
        self.input = input
        self.output = None
        self.solution = None
        self.exit_code = None
        self.state = State.ready  # ready, running, suspended, done, error

        # The following are lists so they can be altered as side effects.
        self.saved_input    = [False]
        self.saved_output   = [False]
        self.saved_solution = [False]

        _thread.start_new_thread(self.run, ())

    def run(self):
        #
        # DO NOT DO ANY GUI STUFF IN HERE, BECAUSE THIS
        # RUNS IN A SEPARATE THREAD!!!
        #
        search_command  = self.program.search_command()
        success_command = self.program.success_command()

        if not search_command or not success_command:
            self.state = State.error
            self.fin  = self.fout = self.ferr = None
        else:
            # use files to avoid buffering problems  (maybe improve later)
            self.fin  = tempfile.TemporaryFile('w+b')  # stdin
            self.fout = tempfile.TemporaryFile('w+b')  # stdout
            self.ferr = tempfile.TemporaryFile('w+b')  # stderr

            self.fin.write(self.input)
            self.fin.seek(0)

            if Win32():
                # creationflag says not to pop a DOS box
                self.process = subprocess.Popen(
                    search_command, stdin=self.fin,
                    stdout=self.fout, stderr=self.ferr,
                    creationflags=win32process.CREATE_NO_WINDOW)
            else:
                self.process = subprocess.Popen(
                    search_command, stdin=self.fin,
                    stdout=self.fout, stderr=self.ferr)
                
            self.state = State.running
            self.exit_code = self.process.wait()  # Wait for process to finish!
            self.state = State.done
            self.fout.seek(0)  # rewind stdout
            self.output = self.fout.read()

            if (self.exit_code == 0 or
                self.program.exists_solution(self.exit_code, self.output)):

                # Extract the solution from stdout
                self.fout.seek(0)
                (rc,output,err) = run_and_wait(success_command, fin=self.fout)

                if rc == 0:  
                    self.solution = output  # at least one solution
                elif rc == 2:
                    self.solution = None  # no solution (clear(print_proofs)?)
                else:
                    self.solution = ('There was an error extracting the %s.' %
                                     self.program.solution_name)

            # Keep files open until self is deleted.

        self.parent.invoke_later(self.parent.job_finished)

    def pause(self):
        if self.state == State.running:
            os.kill(self.process.pid, signal.SIGSTOP)
            self.state = State.suspended

    def resume(self):
        if self.state == State.suspended:
            os.kill(self.process.pid, signal.SIGCONT)
            self.state = State.running

    def get_stderr_info(self):
        if self.state in [State.running, State.suspended, State.done]:
            self.ferr.seek(0)  # rewind
            lines = self.ferr.readlines()
            info = self.program.get_info_from_stderr(lines)
            return info

    def kill(self):
        if self.state == State.running or self.state == State.suspended:
            # Cleanup will occur when the 'run' thread terminates.
            if Win32():
                win32api.TerminateProcess(int(self.process._handle), -1)
            else:
                os.kill(self.process.pid, signal.SIGKILL)

    def done_with_job(self):
        if self.fin:  # if one exists, all exist
            self.fin.close()
            self.fout.close()
            self.ferr.close()
        del self

# end class Run_program()
    
class Program_panel(wx.Panel):

    def __init__(self, parent, program, options):

        self.program = program
        self.job = None
        self.info_panel = None
        self.timer = None        # for monitoring (Info button)

        wx.Panel.__init__(self, parent)
        self.Connect(-1, -1, Invoke_event.my_EVT_INVOKE, self.on_invoke)

        # Surrounding box

        box_program = wx.StaticBox(self, -1, program.box_name)
        box_sizer = wx.StaticBoxSizer(box_program, wx.VERTICAL)

        # Time limit (widget shared with options panel elsewhere)

        id = wx.NewId()
        label_id = wx.NewId()
        opt = copy.deepcopy(options.name_to_opt('max_seconds'))
        if opt:
            opt[Id] = id
            opt[Label_id] = label_id
            opt[Share] = [opt]
            (min, max) = opt[Range]
            options.share_external_option(opt)
            self.time_ctrl = wx.SpinCtrl(self, id, min=min, max=max,
                                         size=(75,-1))
                                         
            self.time_ctrl.SetValue(opt[Default])
        else:
            error_dialog('error sharing max_second option (%s)' % program.name)
            self.time_ctrl = wx.SpinCtrl(self, id, min=-1, max=sys.maxsize,
                                         size=(75,-1))
            self.time_ctrl.SetValue(60)

        self.time_ctrl_opt = opt

        label = wx.StaticText(self, label_id, 'Time Limit: ')
        self.time_ctrl.SetToolTipString('A value of -1 means there is no limit.')
        self.Bind(wx.EVT_SPINCTRL, self.on_time_ctrl, self.time_ctrl)

        time_sizer = wx.BoxSizer(wx.HORIZONTAL)
        time_sizer.Add(label, 0, wx.ALL|wx.ALIGN_CENTER, 1)
        time_sizer.Add(self.time_ctrl, 0, wx.ALL, 1)
        time_sizer.Add(wx.StaticText(self, -1, 'seconds.'), 0,
                       wx.ALL|wx.ALIGN_CENTER, 1)

        # Start, Pause, Kill

        self.start_btn = wx.Button(self, -1, 'Start')
        self.start_btn.SetToolTipString(
            'Start %s with the current input.' % program.name)
        self.Bind(wx.EVT_BUTTON, self.on_start, self.start_btn)
        self.pause_btn = wx.Button(self, -1, 'Pause')
        self.pause_btn.SetToolTipString('Pause or Resume %s.' % program.name)
        self.pause_btn.Enable(False)
        self.Bind(wx.EVT_BUTTON, self.on_pause_resume, self.pause_btn)
        if Win32():
            self.pause_btn.Show(False)
        self.kill_btn = wx.Button(self, -1, 'Kill')
        self.kill_btn.SetToolTipString('Kill %s process.' % program.name)
        self.kill_btn.Enable(False)
        self.Bind(wx.EVT_BUTTON, self.on_kill, self.kill_btn)

        run_sizer = wx.BoxSizer(wx.HORIZONTAL)
        run_sizer.Add(self.start_btn, 0, wx.ALL, 1)
        run_sizer.Add(self.pause_btn, 0, wx.ALL, 1)
        run_sizer.Add(self.kill_btn, 0, wx.ALL, 1)

        # Busy bar

        self.bar = Busy_bar(self, width=200, height=16, delay=100)

        # State line

        width = max_width(list(program.exits.values()), self)
        self.state_text = wx.StaticText(self, -1, 'Ready',
                                        size=(width, -1))

        state_sizer = wx.BoxSizer(wx.HORIZONTAL)
        text = wx.StaticText(self, -1, 'State: ')
        state_sizer.Add(text,            0, wx.ALL|wx.ALIGN_CENTER, 1)
        state_sizer.Add(self.state_text, 1, wx.ALL|wx.ALIGN_CENTER, 1)

        # Info, Show/Save

        self.info_btn = wx.Button(self, -1, 'Info')
        self.info_btn.SetToolTipString(
            'Show some statistics on the %s search.' % program.name)
        self.Bind(wx.EVT_BUTTON, self.on_info, self.info_btn)
        self.info_btn.Enable(False)
        
        self.show_save_btn = wx.Button(self, -1, 'Show/Save')
        if not Mac():
            self.show_save_btn.SetToolTipString(
              'The choices refer to the most recent %s search.' % program.name)
        self.Bind(wx.EVT_BUTTON, self.on_show_save, self.show_save_btn)
        self.show_save_btn.Enable(False)

        show_sizer = wx.BoxSizer(wx.HORIZONTAL)
        show_sizer.Add(self.info_btn,   0, wx.ALL, 1)
        show_sizer.Add(self.show_save_btn, 0, wx.ALL, 1)

        # Overall Layout

        logo_bitmap = program.logo_bitmap()
        if logo_bitmap:
            logo = wx.StaticBitmap(self, -1, logo_bitmap)
            box_sizer.Add(logo,     0, wx.ALL|wx.ALIGN_CENTER,3)
        box_sizer.Add(time_sizer,   0, wx.ALL, 3)
        box_sizer.Add(run_sizer,    0, wx.ALL|wx.ALIGN_CENTER,3)
        box_sizer.Add(self.bar,     0, wx.ALL|wx.ALIGN_CENTER,3)
        box_sizer.Add(state_sizer,  0, wx.ALL|wx.GROW, 3)
        box_sizer.Add(show_sizer,   0, wx.ALL|wx.ALIGN_CENTER, 3)
        self.SetSizer(box_sizer)

    # Methods

    def on_info(self, evt):
        if self.job:
            info = self.job.get_stderr_info()
            self.info_panel = Mini_info(self, 'Info on %s Search' %
                                        self.program.name, info)
            self.info_btn.Enable(False)
            if self.job.state != State.done:
                self.timer = wx.Timer(self, -1)
                wx.EVT_TIMER(self, self.timer.GetId(), self.update_info)
                self.timer.Start(2000)  # milliseconds

    def update_info(self, evt):
        if self.job:
            info = self.job.get_stderr_info()
            self.info_panel.update(info)
            if self.job.state == State.done:
                self.timer.Stop()
                self.timer = None
        else:
            self.timer.Stop()
            self.timer = None

    def info_reset(self):
        if self.timer:
            self.timer.Stop()
            self.timer = None
        self.info_btn.Enable(True)
        self.info_panel = None

    def on_time_ctrl(self, evt): 
        if self.time_ctrl_opt:
            self.time_ctrl_opt[Value] = self.time_ctrl.GetValue()
            update_label(self.time_ctrl_opt)
            update_shared(self.time_ctrl_opt)

    def on_start(self, evt):
        if self.job:
            if self.job.solution and not self.job.saved_solution[0]:
                message = (
                    'There is an unsaved %s.  Delete the %s and continue?' %
                    (self.program.solution_name,self.program.solution_name))
                dlg = wx.MessageDialog(self, message, '',
                                       wx.OK | wx.CANCEL| wx.ICON_QUESTION)
                rc = dlg.ShowModal()
                dlg.Destroy()
                if rc == wx.ID_CANCEL:
                    return

            self.job.done_with_job()
            self.job = None

            self.info_btn.Enable(False)
            if self.info_panel:
                self.info_panel.Close()

        self.start_btn.Enable(False)
        self.time_ctrl.Enable(False)
        self.pause_btn.Enable(True)
        self.kill_btn.Enable(True)
        self.info_btn.Enable(True)
        self.show_save_btn.Enable(False)
        self.bar.start()
        self.state_text.SetLabel('Running')
        input = to_top(self).setup.assemble_input()
        input = 'assign(report_stderr, 2).\n' + input
        self.job = Run_program(self, self.program, input)

    def on_pause_resume(self, evt):
        # assume job is running or suspended
        if self.job.state == State.running:
            self.job.pause()
            self.bar.pause()
            self.pause_btn.SetLabel('Resume')
            self.state_text.SetLabel('Paused')
        elif self.job.state == State.suspended:
            self.job.resume()
            self.bar.resume()
            self.pause_btn.SetLabel('Pause')
            self.state_text.SetLabel('Running')
        else:
            error_dialog('pause_resume: unknown job state')

    def on_kill(self, evt):
        # assume job is running or suspended
        self.job.kill()  # calls job_finished indirectly

    def job_state(self):
        if self.job:
            return self.job.state
        else:
            return None

    def job_finished(self):
        self.bar.stop()
        self.pause_btn.Enable(False)
        self.pause_btn.SetLabel('Pause')
        self.kill_btn.Enable(False)
        self.show_save_btn.Enable(True)
        self.start_btn.Enable(True)
        self.time_ctrl.Enable(True)

        if self.job.state == State.error:
            self.state_text.SetLabel('Program_Not_Found')
            error_dialog('%s binaries not found, looking in\n%s' %
                         (self.program.name, bin_dir()))
        else:
            message = self.program.exit_message(self.job.exit_code)
            self.state_text.SetLabel(message)

            if self.job.exit_code == 1:  # fatal error
                frame = Text_frame(
                    self, to_top(self).box_font,
                    self.program.name + ' Fatal Error',
                    self.job.output,
                    saveas=False)
                frame.hilite_error()
                frame.Show(True)
                error_dialog('A Fatal Error occurred.  The %s output '
                             'is shown.' % self.program.name)
                
            elif self.job.solution:
                self.ss_solution(None)

                if self.job.exit_code != 0:
                    info_dialog('%s Exit: %s. \n%s'
                                % (self.program.name, message,
                                   self.program.some_message))
            elif self.program.exits[self.job.exit_code] != 'Killed':
                info_dialog('%s Exit: %s' % (self.program.name, message))

    def on_show_save(self, evt):
        menu = wx.Menu()

        id = wx.NewId()
        menu.Append(id, self.program.name + ' Input (from most recent search)')
        self.Bind(wx.EVT_MENU, self.ss_input, id=id)

        id = wx.NewId()
        menu.Append(id, self.program.name + ' Output')
        self.Bind(wx.EVT_MENU, self.ss_output, id=id)
        if self.job.state == State.error:
            menu.Enable(id, False)

        id = wx.NewId()
        menu.Append(id, self.program.name + ' ' + self.program.solution_name)
        self.Bind(wx.EVT_MENU, self.ss_solution, id=id)
        if self.job.exit_code != 0 and not self.job.solution:
            menu.Enable(id, False)

        self.PopupMenu(menu)
        menu.Destroy()

    def ss_input(self, evt):
        frame = Text_frame(self, to_top(self).box_font,
                           self.program.name + ' Input',
                           self.job.input,
                           extension='in', saveas=True,
                           saved_flag=self.job.saved_input)
        frame.Show(True)
        
    def ss_output(self, evt):
        frame = Text_frame(self, to_top(self).box_font,
                           self.program.name + ' Output',
                           self.job.output,
                           extension='out', saveas=True,
                           saved_flag=self.job.saved_output)
        frame.Show(True)
        
    def ss_solution(self, evt):
        extra_ops=[('Reformat ...', self.on_reformat)]
        solutions = self.program.count_solutions(self.job.solution)
        if (self.program.name == 'Mace4' and solutions > 1):
            extra_ops.append(('Isofilter...', self.on_isofilter))
        if solutions == 1:
            title = '%s %s' % (self.program.name,self.program.solution_name)
        else:
            title = '%s %s (%d %ss)' % (self.program.name,
                                        self.program.solution_name,
                                        solutions,
                                        self.program.solution_name)
        frame = Text_frame(
            self, to_top(self).box_font,
            title,
            self.job.solution,
            extension=self.program.solution_ext,
            saveas=True,
            saved_flag=self.job.saved_solution,
            extra_operations=extra_ops)
        frame.Show(True)

    def on_reformat(self, evt):
        parent = evt.GetEventObject().GetParent()
        solution = parent.text
        self.program.reformatter(parent, solution, self.job.saved_solution)

    def on_isofilter(self, evt):
        parent = evt.GetEventObject().GetParent()
        solution = parent.text
        frame = Isofilter_frame(self, solution, self.job.saved_solution)

    # The following two methods allow GUI events in the main thread
    # to be initiated by other threads.  See class Invoke_event and
    # the Connect statement in the constructor of this class.

    def on_invoke(self, evt):
        evt.invoke()

    def invoke_later(self, func, *args, **kwargs):
        self.GetEventHandler().AddPendingEvent(Invoke_event(func,args,kwargs))

# END class Program_panel(Panel)

class Control_panel(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent, style=wx.FULL_REPAINT_ON_RESIZE)

        # Show Input Button

        self.show_input_btn = wx.Button(self, -1, 'Show Current Input')
        self.show_input_btn.SetToolTipString(
            'Show the data in the setup panel as a text file.\n'
            'This is what will be given to Prover9 and/or Mace4.\n'
            'This need not be saved before running Prover9 or Mace4.')
        self.Bind(wx.EVT_BUTTON, self.show_input, self.show_input_btn)

        # Program Panels

        self.prover9 = Program_panel(self, Prover9(),
                                     to_top(self).setup.p9_options.panels)
        self.mace4 = Program_panel(self, Mace4(),
                                   to_top(self).setup.m4_options)

        # Overall Layout

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.show_input_btn, 0, wx.ALL|wx.ALIGN_CENTER, 5)
        sizer.Add((1,10), 1)
        sizer.Add(self.prover9, 0, wx.GROW, 5)
        sizer.Add((1,10), 1)
        sizer.Add(self.mace4, 0, wx.GROW, 5)
        sizer.Add((1,10), 1)
        self.SetSizer(sizer)
        sizer.Fit(self)

    def show_input(self, evt):
        top = to_top(self)
        input = top.setup.assemble_input()
        frame = Text_frame(self, top.box_font,
                           'Current Input', input,
                           extension='in', saveas=False)
        frame.Show(True)

# END class Control_panel(Panel)

class Isofilter_frame(wx.Frame):
    def __init__(self, parent, models, saved_solution):

        self.parent = parent
        self.models = models
        self.state = State.ready  # We'll use ready and running
        
        wx.Frame.__init__(self, parent, title='Isofilter',
                          pos=pos_for_center((0,0)))

        self.Connect(-1, -1, Invoke_event.my_EVT_INVOKE, self.on_invoke)

        ops_string = ' '.join(ops_in_interp(models))

        check_lab  = wx.StaticText(self, -1, 'Check:')
        self.check_id = wx.NewId()
        self.check_ctrl = wx.TextCtrl(self, self.check_id,
                                      value=ops_string, size=(200,-1))

        out_lab  = wx.StaticText(self, -1, 'Output:')
        self.out_id = wx.NewId()
        self.out_ctrl = wx.TextCtrl(self, self.out_id,
                                    value=ops_string, size=(200,-1))

        self.ignore_id = wx.NewId()
        self.ignore_cb = wx.CheckBox(self, self.ignore_id, 'Ignore Constants')

        self.wrap_id = wx.NewId()
        self.wrap_cb = wx.CheckBox(self, self.wrap_id, 'Enclose in List')

        alg_lab =  wx.StaticText(self, -1, 'Algorithm:')
        self.alg_id = wx.NewId()
        self.alg_ch = wx.Choice(self, self.alg_id,
                                choices=['Occurrence Profiles',
                                         'Canonical Forms'])
        self.alg_ch.SetSelection(0)

        self.start_btn = wx.Button(self, -1, 'Start')
        self.Bind(wx.EVT_BUTTON, self.on_start, self.start_btn)

        cancel_btn = wx.Button(self, -1, 'Cancel')
        self.Bind(wx.EVT_BUTTON, self.on_cancel, cancel_btn)

        self.bar = Busy_bar(self, width=200, height=16, delay=100)

        gsizer = wx.GridBagSizer(5,5)
        gsizer.Add(alg_lab,         (0,0))
        gsizer.Add(self.alg_ch,     (0,1))
        gsizer.Add(check_lab,       (1,0))
        gsizer.Add(self.check_ctrl, (1,1))
        gsizer.Add(out_lab,         (2,0))
        gsizer.Add(self.out_ctrl,   (2,1))

        bsizer = wx.BoxSizer(wx.HORIZONTAL)
        bsizer.Add(self.start_btn)
        bsizer.Add(cancel_btn)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add((1,5))
        sizer.Add(gsizer, 0, wx.ALL|wx.ALIGN_CENTER, 5)
        sizer.Add(self.ignore_cb, 0, wx.ALL, 5)
        sizer.Add(self.wrap_cb, 0, wx.ALL, 5)
        sizer.Add(bsizer, 0, wx.ALL|wx.ALIGN_CENTER, 5)
        sizer.Add(self.bar, 0, wx.ALL|wx.ALIGN_CENTER, 5)

        self.SetSizer(sizer)
        sizer.Fit(self)
        self.Show(True)

    # The following two methods allow GUI events in the main thread
    # to be initiated by other threads.  See class Invoke_event and
    # the Connect statement in the constructor of this class.

    def on_invoke(self, evt):
        evt.invoke()

    def invoke_later(self, func, *args, **kwargs):
        self.GetEventHandler().AddPendingEvent(Invoke_event(func,args,kwargs))

    def on_start(self, evt):
        if self.alg_ch.GetStringSelection() == 'Occurrence Profiles':
            command = isofilter_command('isofilter')  # returns list
        else:
            command = isofilter_command('isofilter2')  # returns list
        if command == None:
            error_dialog('Isofilter binary not found.')
            self.Close()
        else:
            if self.wrap_cb.IsChecked():
                command.append('wrap')
            if self.ignore_cb.IsChecked():
                command.append('ignore_consants')
            check = self.check_ctrl.GetValue()
            if check.strip() != '':
                command.append('check')
                command.append(check)
            out = self.out_ctrl.GetValue()
            if out.strip() != '':
                command.append('output')
                command.append(out)

            self.command = command
            self.start_btn.Disable()
            self.bar.start()
            _thread.start_new_thread(self.run, ())

    def run(self):
        #
        # DO NOT DO ANY GUI STUFF IN HERE, BECAUSE THIS
        # RUNS IN A SEPARATE THREAD!!!
        #

        self.fin  = tempfile.TemporaryFile('w+b')  # stdin
        self.fout = tempfile.TemporaryFile('w+b')  # stdout
        self.ferr = tempfile.TemporaryFile('w+b')  # stderr

        self.fin.write(self.models)
        self.fin.seek(0)

        if Win32():
            # creationflag says not to pop a DOS box
            self.process = subprocess.Popen(
                self.command, stdin=self.fin,
                stdout=self.fout, stderr=self.ferr,
                creationflags=win32process.CREATE_NO_WINDOW)
        else:
            self.process = subprocess.Popen(
                self.command, stdin=self.fin,
                stdout=self.fout, stderr=self.ferr)

        self.state = State.running
        self.exit_code = self.process.wait()  # Wait for process to finish!
        self.fout.seek(0)  # rewind stdout
        self.filtered_models = self.fout.read()

        self.invoke_later(self.job_finished)

    def job_finished(self):
        self.bar.stop()
        if self.exit_code == 1:
            self.ferr.seek(0)  # rewind stderr
            err = self.ferr.read()
            error_dialog('Isofilter error:\n\n' + err)
        elif self.exit_code == 0:
            n = self.filtered_models.rfind(': input=')
            if n == -1:
                error_dialog('Isofilter error')
            else:
                x = re.split('=|,', self.filtered_models[n:])
                input = int(x[1])
                kept = int(x[3])
   
                extra_ops=[('Reformat ...', self.parent.on_reformat)]

                args = ' '.join(self.command[1:])

                frame = Text_frame(
                    self.parent, to_top(self).box_font,
                    'Isofilter Result (%s, %d models)' % (args, kept),
                    self.filtered_models,
                    extension='model',
                    saveas=True,
                    off_center=20,
                    saved_flag=self.parent.job.saved_solution,
                    extra_operations=extra_ops)
                frame.Show(True)

                info_dialog(('Isofilter received %d models, eliminated %d, '
                             'giving %d nonisomorphic model(s).') %
                            (input, input-kept, kept))

        self.fin.close()
        self.fout.close()
        self.ferr.close()
        self.Close()
        
    def on_cancel(self, evt):
        if self.state == State.running:
            # Cleanup will occur when the 'run' thread terminates.
            if Win32():
                win32api.TerminateProcess(int(self.process._handle), -1)
            else:
                os.kill(self.process.pid, signal.SIGKILL)
        else:
            self.Close()
    
# END class Isofilter_frame(wx.Frame)

def syntax_check(input):

    prover9_command  = Prover9().search_command()

    if not prover9_command:
        error_dialog("syntax check binary (prover9) not found")
    else:
        # use files to avoid buffering problems  (maybe improve later)

        input2 = 'assign(max_given,0).\n ' + input

        (exit_code,output,err) = run_and_wait(prover9_command, input=input2)
        
        exit_str = Prover9().exit_message(exit_code)

        if exit_str in ['Killed', 'Crashed', 'Interruppted']:
            return (exit_str, output, None)
        elif exit_code != 1:  # Fatal error
            return ('Okay', None, None)
        else:
            if re.search('%%ERROR', output):
                r = re.compile('(?<=%%ERROR: ).*')
                m = r.search(output)
                message = output[m.start():m.end()-1] + '.'
                r = re.compile('(?<=%%START ERROR%%).*(?=%%END ERROR%%)',
                               re.DOTALL)  # allow to cross lines
                m = r.search(output)
                if m:
                    error = output[m.start():m.end()].strip()
                else:
                    error = None
                return ('Input_Error', message, error)
            else:
                return ('Other_Error', output, None)

# END def syntax_check




#     Copyright (C) 2007 William McCune
#
#     This file is part of the LADR Deduction Library.
#
#     The LADR Deduction Library is free software; you can redistribute it
#     and/or modify it under the terms of the GNU General Public License
#     as published by the Free Software Foundation; either version 2 of the
#     License, or (at your option) any later version.
#
#     The LADR Deduction Library is distributed in the hope that it will be
#     useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with the LADR Deduction Library; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
#

# system imports

import os, re, wx

# local imports

from platforms import *

class State:
    """
    For various processes and threads.
    """
    ready     = 0
    running   = 1
    suspended = 2
    done      = 3
    error     = 4

def to_top(w):
    while w.GetParent():
        w = w.GetParent()
    return w

def absolute_position(w):
    (x,y) = w.GetPosition()
    if not w.GetParent():
        return (x,y)
    else:
        (a,b) = absolute_position(w.GetParent())
        return (x+a, y+b)

def size_that_fits(recommended_size):
    (r_width, r_height) = recommended_size
    screen_width  = wx.SystemSettings.GetMetric(wx.SYS_SCREEN_X)
    screen_height  = wx.SystemSettings.GetMetric(wx.SYS_SCREEN_Y)
    return (min(r_width, screen_width), min(r_height, screen_height))

def pos_for_center(size):
    (frame_width, frame_height) = size
    screen_width  = wx.SystemSettings.GetMetric(wx.SYS_SCREEN_X)
    screen_height  = wx.SystemSettings.GetMetric(wx.SYS_SCREEN_Y)
    x = screen_width/2 - frame_width/2
    y = screen_height/2 - frame_height/2
    return (max(x,0),max(y,0))

def center_of_screen():
    screen_width  = wx.SystemSettings.GetMetric(wx.SYS_SCREEN_X)
    screen_height  = wx.SystemSettings.GetMetric(wx.SYS_SCREEN_Y)
    return (screen_width/2, screen_height/2)

def error_dialog(message):
    dlg = wx.MessageDialog(None, message, '', wx.OK | wx.ICON_ERROR)
    dlg.ShowModal()
    dlg.Destroy()

def info_dialog(message):
    dlg = wx.MessageDialog(None, message, '', wx.OK | wx.ICON_INFORMATION)
    dlg.ShowModal()
    dlg.Destroy()

def open_dir_style(current_path):
    # Mac and Win32 remember directory, even after quitting program;
    # else (GTK), use our
    style = wx.OPEN
    if Mac() or Win32():
        return ('', style)
    else:
        # return (os.getcwd(), wx.OPEN | wx.CHANGE_DIR)
        if current_path:
            return (os.path.dirname(current_path), style)
        else:
            return (os.getcwd(), style)

def saveas_dir_style(current_path):
    style = wx.SAVE | wx.OVERWRITE_PROMPT
    if Win32():  # Win32 uses dir remembered from 'open'
        return ('', style)
    elif Mac():  # Mac doesn't use dir remembered from 'open'
        if current_path:
            return (os.path.dirname(current_path), style)
        else:
            return ('', wx.SAVE)
    else:        # GTK doesn't remember
        # return (os.getcwd(), style | wx.CHANGE_DIR)
        if current_path:
            return (os.path.dirname(current_path), style)
        else:
            return (os.getcwd(), style)

def max_width(strings, window):
    max_wid = 0
    for s in strings:
        width = window.GetTextExtent(s)[0]
        max_wid = max(max_wid, width)
    return max_wid

class Text_frame(wx.Frame):
    def __init__(self, parent, font, title, text,
                 extension=None, saveas=True,
                 off_center=0, saved_flag=None,
                 extra_operations=[]):
        
        size = size_that_fits((900,650))     # reduce if screen too small
        (x,y) = pos_for_center(size)         # position to center frame
        pos = (x+off_center, y+off_center)

        wx.Frame.__init__(self, parent, title=title, size=size, pos=pos)

        self.extension = extension
        self.saved_flag = saved_flag
        self.text = text

        if saveas:
            saveas_btn = wx.Button(self, -1, 'Save as...')
            self.Bind(wx.EVT_BUTTON, self.on_saveas, saveas_btn)

        extra_btns = []
        for (label,func) in extra_operations:
            btn = wx.Button(self, -1, label)
            self.Bind(wx.EVT_BUTTON, func, btn)
            extra_btns.append(btn)

        close_btn = wx.Button(self, -1, 'Close')
        self.Bind(wx.EVT_BUTTON, self.on_close, close_btn)

        self.txt = wx.TextCtrl(self,
                               style=wx.TE_MULTILINE|wx.TE_READONLY|wx.HSCROLL|
                               wx.TE_RICH2)  # TE_RICH2 allows > 32K in Win32
        self.txt.SetFont(font)
        
        self.txt.AppendText(text)
        self.txt.ShowPosition(0)
 
        sub_sizer = wx.BoxSizer(wx.HORIZONTAL)
        if saveas:
            sub_sizer.Add(saveas_btn, 0, wx.ALL, 3)
        for btn in extra_btns:
            sub_sizer.Add(btn, 0, wx.ALL, 3)
        sub_sizer.Add((0,0), 1)  # strechable space
        sub_sizer.Add(close_btn, 0, wx.ALL, 3)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(sub_sizer, 0, wx.ALL|wx.GROW, 3)
        sizer.Add(self.txt, 1, wx.ALL|wx.GROW, 3)
        self.SetSizer(sizer)

    def append(self, text):
        self.txt.AppendText(text)
        self.txt.ShowPosition(self.txt.GetLastPosition())

    def on_saveas(self, evt):
        (dir,style) = saveas_dir_style(to_top(self).current_path)

        if to_top(self).current_path and self.extension:
            dfile = os.path.basename(to_top(self).current_path)
            dfile = re.sub('\.[^.]*$', '', dfile)  # get rid of any extension
            dfile = '%s.%s' % (dfile,self.extension)    # append new extension
        else:
            dfile = ''

        dlg = wx.FileDialog(self, message='Save file as ...',
                            defaultDir=dir, defaultFile=dfile, style=style)
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()      # full path
            try:
                f = open(path, 'w')
                f.write(self.txt.GetValue())
                # Do not update to_top(self).current_path
                if self.saved_flag:
                    self.saved_flag[0] = True
            except IOError as e:
                error_dialog('Error opening file %s for writing.' % path)

        dlg.Destroy()

    def on_close(self, evt):
        self.Close()

    def hilite_error(self):
        start = self.txt.GetValue().find('%%START ERROR%%')
        if start > 0:
            end = self.txt.GetValue().find('%%END ERROR%%', start)
            if end > 0:
                self.txt.SetStyle(start+15, end,
                                  wx.TextAttr('RED',
                                              wx.Colour(200,200,255)))
        else:
            start = self.txt.GetValue().find('%%ERROR:')
            if start > 0:
                end = self.txt.GetValue().find('\n', start)
                if end > 0:
                    self.txt.SetStyle(start+8, end,
                                      wx.TextAttr('RED',
                                                  wx.Colour(200,200,255)))
            

# END class Text_frame(wx.Frame)

class Mini_info(wx.MiniFrame):
    def __init__(self, parent, title, items):
        
        # pos = center_of_screen()
        pos = absolute_position(parent)

        if Win32():
            style = wx.STAY_ON_TOP|wx.DEFAULT_FRAME_STYLE
        elif Mac():
            style = wx.STAY_ON_TOP|wx.CAPTION|wx.CLOSE_BOX
        else:
            style = wx.STAY_ON_TOP   # GTK ignores style for MiniFrame

        wx.MiniFrame.__init__(self, parent, title=title, pos=pos, style=style)

        # Cannot get a close button on titlebar in GTK.
        if Win32() or Mac():
            close_btn = None
        else:
            close_btn = wx.Button(self, -1, 'Close', style=wx.BU_EXACTFIT)
            self.Bind(wx.EVT_BUTTON, self.on_close, close_btn)
        self.Bind(wx.EVT_CLOSE, self.on_close)

        gsizer = wx.GridSizer(len(items), 2)

        self.val_labels = []
        for (name,val) in items:
            name_lab = wx.StaticText(self, -1, name + ':')
            val_lab  = wx.StaticText(self, -1, str(val), size=(75,-1),
                                     style=wx.ALIGN_RIGHT|wx.ST_NO_AUTORESIZE)
            self.val_labels.append(val_lab)
            gsizer.Add(name_lab, 0, wx.ALIGN_LEFT, 3)
            gsizer.Add(val_lab, 0, wx.ALIGN_RIGHT, 3)

        sizer = wx.BoxSizer(wx.VERTICAL)
        if close_btn:
            sizer.Add(close_btn, 0, wx.ALL|wx.ALIGN_RIGHT, 3)
        sizer.Add(gsizer, 0, wx.ALL|wx.ALIGN_CENTER, 3)
        self.SetSizer(sizer)
        self.Fit()
        self.Show()

    def on_close(self, evt):
        self.GetParent().info_reset()
        self.Destroy()

    def update(self, items):
        i = 0
        for (_,val) in items:
            lab = self.val_labels[i]
            lab.SetLabel(str(val))
            i += 1
        self.Fit()
        
# END class Mini_info(wx.MiniFrame)

class Invoke_event(wx.PyEvent):
    """
    This class is used so that a side thread can invoke
    a function in the main GUI thread.
    """
    my_EVT_INVOKE = wx.NewEventType()

    def __init__(self, func, args, kwargs):
        wx.PyEvent.__init__(self)
        self.SetEventType(Invoke_event.my_EVT_INVOKE)
        self.__func = func
        self.__args = args
        self.__kwargs = kwargs

    def invoke(self):
        self.__func(*self.__args, **self.__kwargs)

# END class Invoke_event(PyEvent)

class Busy_bar(wx.Gauge):

    def __init__(self, parent, width=200, height=16,
                 pixels_to_move=2, fill=.9, delay=200):

        wx.Gauge.__init__(self, parent, range=100, size=(width,height))

        self.delay = delay
        self.range = range
        self.state = State.ready
        self.position = 40
        self.direction = 1

    def update_bar(self, evt):
        # This is run periodically, triggered by the timer in self.start().

        self.position += self.direction
        if self.position >= 60:
            self.direction = -1
        elif self.position <= 40:
            self.direction = 1

        self.SetValue(self.position)

    def start(self):
        self.timer = wx.Timer(self, -1)
        wx.EVT_TIMER(self, self.timer.GetId(), self.update_bar)
        self.timer.Start(self.delay)
        self.state = State.running

    def pause(self):
        self.timer.Stop()
        self.state = State.suspended

    def resume(self):
        self.start()
        
    def stop(self):
        self.timer.Stop()
        self.SetValue(0)
        self.position = 40
        self.direction = 1
        self.state = State.ready

# END class Busy_bar(Guage)




#     Copyright (C) 2007 William McCune
#
#     This file is part of the LADR Deduction Library.
#
#     The LADR Deduction Library is free software; you can redistribute it
#     and/or modify it under the terms of the GNU General Public License
#     as published by the Free Software Foundation; either version 2 of the
#     License, or (at your option) any later version.
#
#     The LADR Deduction Library is distributed in the hope that it will be
#     useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with the LADR Deduction Library; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
#

# system imports

import os, sys, imp

# local imports

from platforms import *

def path_info():
    info = ('os.getcwd(): %s\n'
            'sys.argv[0]: %s\n'
            'sys.path[0]: %s\n'
            'os.path.dirname(sys.executable): %s'
            'os.path.dirname(os.path.abspath(sys.argv[0])): %s' %
            (os.getcwd(),
             sys.argv[0],
             sys.path[0],
             os.path.dirname(sys.executable),
             os.path.dirname(os.path.abspath(sys.argv[0])),
            ))
    return info

def program_dir():
    """
    This gets the full pathname of the directory containing the program.
    It is used for referring to other files (binaries, images, etc.).
    """
    if (Win32() and (hasattr(sys, 'frozen') or imp.is_frozen('__main__'))):
        # running from exe generated by py2exe
        return os.path.dirname(sys.executable)
    else:
        return sys.path[0]
        # return os.path.dirname(os.path.abspath(sys.argv[0]))

def bin():
    if Win32():
        return 'bin-win32'
    elif Mac():
        if Mac_ppc():
            return 'bin-mac-ppc'
        else:
            return 'bin-mac-intel'
    else:
        return 'bin'

def bin_dir():
    return '/usr/local/libexec/prover9/'

def image_dir():
    return '/usr/local/share/p9m4/Images/'

def sample_dir():
    return '/usr/local/share/p9m4/Samples/'

def binary_ok(fullpath):
    if not fullpath:
        return False
    elif Win32():
        return os.access(fullpath + '.exe', os.X_OK)
    else:
        return os.access(fullpath, os.X_OK)




#     Copyright (C) 2007 William McCune
#
#     This file is part of the LADR Deduction Library.
#
#     The LADR Deduction Library is free software; you can redistribute it
#     and/or modify it under the terms of the GNU General Public License
#     as published by the Free Software Foundation; either version 2 of the
#     License, or (at your option) any later version.
#
#     The LADR Deduction Library is distributed in the hope that it will be
#     useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with the LADR Deduction Library; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
#

# system imports

import os, sys
import re
import wx

# local imports

import partition_input
import utilities
from files import *
from control import *
from platforms import *
from wx_utilities import *
from options import *

# When saving an input file, a few comments are added; when
# opening a saved input file, those comments are removed.

Comment_banner  = '% Saved by.*\n'
Comment_opt_dep = '% GUI handles dependencies'
Comment_lang    = '% Language Options'
Comment_p9_opt  = '% Options for Prover9'
Comment_m4_opt  = '% Options for Mace4'
Comment_p9_add  = '% Additional input for Prover9'
Comment_m4_add  = '% Additional input for Mace4'

All_added_comments = [Comment_banner, Comment_opt_dep, Comment_lang,
                      Comment_p9_opt, Comment_m4_opt,
                      Comment_p9_add, Comment_p9_add]

class Input_panel(wx.Panel):

    def __init__(self, parent, title, auto_highlight):
        self.title = title
        self.have_new_text = False
        wx.Panel.__init__(self, parent)

        title_display = title + ':'
        width = max_width([title_display], self) + 10  # +10 prevents wrap
        text = wx.StaticText(self, -1, title + ':', size=(width,-1))
        wff_btn = wx.Button(self, -1, 'Well Formed?')
        wff_btn.SetToolTipString(
            'Check syntax of %s.  If there are multiple '
            'errors, only the first will be shown.' % title)
        self.Bind(wx.EVT_BUTTON, self.well_formed_check, wff_btn)
        self.hilite_btn = wx.Button(self, -1, 'Highlight')
        self.hilite_btn.SetToolTipString(
            'Color comments and attributes in this box now. '
            'This button is a hack because on some platforms, '
            'the automatic highlighting causes an annoying '
            'flash every few seconds.')
        self.Bind(wx.EVT_BUTTON, self.on_hilite, self.hilite_btn)
        clear_btn = wx.Button(self, -1, 'Clear')
        clear_btn.SetToolTipString('Clear ' + title)
        self.Bind(wx.EVT_BUTTON, self.clear, clear_btn)

        sub_sizer = wx.BoxSizer(wx.HORIZONTAL)
        sub_sizer.Add(text, 1, wx.ALIGN_BOTTOM, 0)
        sub_sizer.Add(self.hilite_btn, 0, wx.ALL, 3)
        sub_sizer.Add(wff_btn, 0, wx.ALL, 3)
        sub_sizer.Add(clear_btn, 0, wx.ALL, 3)

        self.ed = wx.TextCtrl(self,  # Win32 neews TE_RICH2 for color
                              style=wx.TE_MULTILINE|wx.HSCROLL|wx.TE_RICH2)
        if Win32():
            self.ed.Bind(wx.EVT_CHAR, self.on_char, self.ed)  # bad on Mac()
        else:
            self.ed.Bind(wx.EVT_TEXT, self.on_text, self.ed)  # bad on Win32()

        self.ed.SetFont(to_top(self).box_font)
        self.ed.SetDefaultStyle(wx.TextAttr('BLACK','WHITE',
                                            to_top(self).box_font))

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(sub_sizer, 0, wx.ALL|wx.GROW, 3)
        sizer.Add(self.ed, 1, wx.ALL|wx.GROW, 3)
        self.SetSizer(sizer)

        self.timer = None
        if auto_highlight:
            self.start_auto_highlight()
        else:
            self.hilite_btn.Show(True)
            self.Layout()

    def start_auto_highlight(self):
        if self.timer:
            error_dialog('start_auto_highlight: timer alreay exists')
        else:
            self.timer = wx.Timer(self, -1)
            wx.EVT_TIMER(self, self.timer.GetId(), self.check_highlight)
            self.timer.Start(2000)  # check every 2 seconds
            self.hilite_btn.Show(False)
        
    def stop_auto_highlight(self):
        if self.timer:
            self.timer.Stop()
            self.timer = None
            self.hilite_btn.Show(True)
            self.Layout()
        else:
            error_dialog('stop_auto_highlight: timer not found')

    def on_hilite(self, evt):
        self.highlight()
        if Mac():
            # There should be a better way to do this.
            self.Show(False)
            self.Show(True)

    def well_formed_check(self, evt):
        if self.title == 'Assumptions':
            head = '\nformulas(assumptions).\n'
            tail = '\nend_of_list.\n'
        elif self.title == 'Goals':
            head = '\nformulas(goals).\n'
            tail = '\nend_of_list.\n'
        else:
            head = tail = ''
        
        text = self.ed.GetValue()
        if self.title == 'Language Options':
            lang_opt = ''  # this may be temporary, until language panel is done
        else:
            lang_opt = to_top(self).setup.language.get_language_input()

        input = ('%s%s%s%s' % (lang_opt, head, text, tail))

        (exit, message, error) = syntax_check(input)

        if exit == 'Okay':
            info_dialog('This part of the input looks good!')
        elif exit == 'Input_Error':
            if error:
                start = text.find(error)
                if (start >= 0):
                    end = start + len(error)
                    self.ed.SetStyle(start, end,
                                     wx.TextAttr('RED',
                                                 wx.Colour(200,200,255)))
            error_dialog('%s\n%s' % (message,error if error else ''))
        else:
            frame = Text_frame(self, to_top(self).box_font,
                               'Error Output',
                               message, saveas=False)
            frame.Show(True)
            error_dialog('Unknown error; see the \'Error Output\' window.')

    def clear(self, evt):
        self.ed.Clear()

    def highlight(self):
        str = self.ed.GetValue()
        # reset to all black text
        self.ed.SetStyle(0, self.ed.GetLastPosition(),
                         wx.TextAttr('BLACK', 'WHITE', to_top(self).box_font))

        # attributes:
        spans = utilities.pattern_spans('#[^.\n]*[.\n]', str)
        for (start,end) in spans:
            self.ed.SetStyle(start, end, wx.TextAttr(wx.Colour(0,0,200)))

        font = self.ed.GetFont()
        font.SetStyle(wx.FONTSTYLE_ITALIC)

        # comments (line and block)
        spans = utilities.comment_spans(str)
        for (start,end) in spans:
            self.ed.SetStyle(start, end, wx.TextAttr(wx.Colour(0,160,0),
                                                     font=font))

        # following needed on mac to undo italics (???)
        self.ed.SetDefaultStyle(wx.TextAttr('BLACK','WHITE',
                                            to_top(self).box_font))
        
    def on_text(self, evt):
        # This gets called whenever the text is changed (EVT_TEXT)
        self.have_new_text = True

    def on_char(self, evt):
        # This gets called whenever a character is inserted (EVT_CHAR)
        self.have_new_text = True
        evt.Skip() # allows normal processing of char

    def check_highlight(self, evt):
        # This gets called periodically.
        if self.have_new_text:
            self.have_new_text = False
            self.timer.Stop()
            self.highlight()
            self.timer.Start(2000)

# END class Input_panel(Panel)

class P9_options_panel(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent)

        panel1 = wx.Panel(self)
        panel1.SetBackgroundColour('white')
        self.panel2 = wx.Panel(self)

        self.type = ['Basic Options', 'All Options']

        logo_bitmap = Prover9().logo_bitmap()
        if logo_bitmap:
            heading = wx.StaticBitmap(panel1, -1, logo_bitmap)
        else:
            heading = wx.StaticText(panel1, -1, 'Prover9')

        self.rb1 = wx.RadioBox(panel1, -1, '', choices=self.type,
                               majorDimension=1, style=wx.RA_SPECIFY_COLS)
        self.rb1.SetToolTip(wx.ToolTip(
            'Show only basic options, or enable access to all options. '
            'Note that "Basic Options" is a subset of "All Options", and '
            'different groups in "All Options" may share options. '
            'Changing one of the widgets for a shared option will '
            'cause other widgets for that option to be updated in kind.'))

        self.panels = P9_options(self.panel2)  # dictionary indexed by sets
        self.sets = self.panels.optionset_names() # 'Basic Options' is first

        self.rb2 = wx.RadioBox(panel1, -1, 'Option Groups',
                               choices=self.sets[1:],  # all but first
                               majorDimension = 1, style=wx.RA_SPECIFY_COLS)
        self.rb2.SetToolTip(wx.ToolTip('These are all of the Prover9 '
                                       'options, separated into groups.'))
        self.rb2.Enable(False)

        self.Bind(wx.EVT_RADIOBOX, self.handle_rb1, self.rb1)
        self.Bind(wx.EVT_RADIOBOX, self.handle_rb2, self.rb2)

        self.reset = wx.Button(panel1, -1, 'Reset All to Defaults')
        self.Bind(wx.EVT_BUTTON, self.on_reset, self.reset)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add((1,10), 0)
        # sizer.Add(heading, 0, wx.LEFT|wx.RIGHT|wx.GROW|wx.ALIGN_CENTER, 10)
        sizer.Add(heading, 0, wx.ALL|wx.ALIGN_CENTER, 0)
        sizer.Add((1,5), 0)
        sizer.Add(self.rb1, 0, wx.LEFT|wx.RIGHT|wx.GROW, 10)
        sizer.Add((1,10), 0)
        sizer.Add(self.rb2, 0, wx.LEFT|wx.RIGHT|wx.GROW, 10)
        sizer.Add((1,10), 0)
        sizer.Add(self.reset, 0, wx.LEFT|wx.RIGHT|wx.GROW, 10)
        panel1.SetSizer(sizer)

        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(panel1, 0, wx.ALL|wx.GROW, 3)
        sizer.Add(self.panel2, 1, wx.ALL|wx.GROW, 3)
        self.SetSizer(sizer)

        self.current_option_set = None
        self.switch_options(0)

    def switch_options(self, item):
        if self.current_option_set:
            self.current_option_set.Show(False)
        self.current_option_set = self.panels.panels[self.sets[item]]
        self.current_option_set.Show(True)
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add((0,0), 1)
        sizer.Add(self.current_option_set, 0, wx.ALIGN_CENTER, 10)
        sizer.Add((0,0), 3)
        self.panel2.SetSizer(sizer)
        self.panel2.Layout()

    def handle_rb1(self, evt):
        item = evt.GetInt()
        if self.type[item] == 'Basic Options':
            self.switch_options(0)
            self.rb2.Enable(False)
        else:
            item = self.rb2.GetSelection()
            self.switch_options(item+1)
            self.rb2.Enable(True)
        self.Layout()         # rearrange children

    def handle_rb2(self, evt):
        item = evt.GetInt()
        self.switch_options(item+1)

    def on_reset(self, evt):
        self.panels.reset()

# END class P9_options_panel(Panel)

class Language_panel(wx.Panel):
    def __init__(self, parent, options):
        wx.Panel.__init__(self, parent)

        # Directions

        temp_help = (
        'Some day, this panel will have widgets for specifying all of '
        'the language options.  Until then, if you know the '
        'Prover9/Mace4 commands for doing so, you can give them '
        'in the following text box.  The commands include '
        '"op", for specifying infix/prefix/postfix and precedence, '
        'and "redeclare", for changing the the symbols with built-in '
        'semantics.  For example,\n\n'
        '   op(450, infix, "@").\n'
        '   redeclare(implication, IMPLIES).')

        # directions = wx.StaticText(self, -1, temp_help)
        directions = wx.TextCtrl(self, -1, temp_help, style=wx.TE_MULTILINE)

        # Prolog-Style Variables (Shared with other options widgets)
        
        id = wx.NewId()
        label_id = wx.NewId()
        opt = copy.deepcopy(options.name_to_opt('prolog_style_variables'))
        if opt:
            opt[Id] = id
            opt[Label_id] = id  # we're not using a separate label here
            opt[Share] = [opt]  # note circular structure
            options.share_external_option(opt)
            self.prolog_cb = wx.CheckBox(self, id, 'Prolog-Style Variables')
            self.prolog_cb.SetValue(opt[Default])
            tip = opt[Tip]
        else:
            error_dialog('error sharing prolog_style_variables option')
            self.prolog_cb = wx.CheckBox(self, id, 'Prolog-Style Variables')
            self.prolog_cb.SetValue(False)
            tip = ''

        self.prolog_cb_opt = opt

        self.Bind(wx.EVT_CHECKBOX, self.on_prolog, self.prolog_cb)
        self.prolog_cb.SetToolTipString(tip)

        # Text Box

        self.input = Input_panel(self, 'Language Options',
                                 to_top(self).auto_highlight())

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.prolog_cb, 0, wx.ALL|wx.GROW, 10)
        sizer.Add(directions, 0, wx.ALL|wx.GROW, 10)
        sizer.Add(self.input, 1, wx.ALL|wx.GROW, 1)

        self.SetSizer(sizer)

    def on_prolog(self, evt):
        if self.prolog_cb_opt:
            self.prolog_cb_opt[Value] = self.prolog_cb.GetValue()
            update_label(self.prolog_cb_opt)
            update_shared(self.prolog_cb_opt)

    def get_language_input(self):
        return self.input.ed.GetValue()

# end class Language_panel(wx.Panel):

class Setup_tabs(wx.Notebook):
    def __init__(self, parent):
        wx.Notebook.__init__(self, parent,
                             style=wx.NO_BORDER|wx.FULL_REPAINT_ON_RESIZE|wx.NB_NOPAGETHEME)

        # self.Bind(wx.EVT_NOTEBOOK_PAGE_CHANGED, self.on_changed)

        # Formulas Tab

        self.formulas = wx.SplitterWindow(self, -1)
        self.assumps = Input_panel(self.formulas, 'Assumptions',
                                 to_top(self).auto_highlight())
        self.goals   = Input_panel(self.formulas, 'Goals',
                                 to_top(self).auto_highlight())

        self.formulas.SplitHorizontally(self.assumps, self.goals)
        self.formulas.SetSashGravity(0.75)
        self.formulas.SetMinimumPaneSize(150)
        if Mac() or Win32():
            self.formulas.SetSashPosition(450)

        # Prover9 Options Tab

        self.p9_options = P9_options_panel(self)

        # Mace4 Options Tab

        self.m4_options = M4_options(self, Mace4().logo_bitmap())

        # self.m4_options.output_transformed()
        # self.p9_options.panels.output_transformed()

        link_options_by_names(self.p9_options.panels, self.m4_options,
                              ['prolog_style_variables'])

        # Additional Input Tab

        self.add = wx.SplitterWindow(self, -1)
        self.add_p9 = Input_panel(self.add, 'Additional Input for Prover9',
                                 to_top(self).auto_highlight())
        self.add_m4 = Input_panel(self.add, 'Additional Input for Mace4',
                                 to_top(self).auto_highlight())
        self.add.SplitHorizontally(self.add_p9, self.add_m4)
                                   
        self.add.SetSashGravity(0.75)
        self.add.SetMinimumPaneSize(150)
        if Mac() or Win32():
            self.formulas.SetSashPosition(450)

        # Language Tab (construction assumes options tabs exist)

        self.language = Language_panel(self, self.m4_options)

        # Add tabs to Notebook

        self.AddPage(self.language, 'Language Options')
        self.AddPage(self.formulas, 'Formulas')
        self.AddPage(self.p9_options, 'Prover9 Options')
        self.AddPage(self.m4_options.panel, 'Mace4 Options')
        self.AddPage(self.add, 'Additional Input')

        self.SetSelection(1)  # Start with second page (Formulas) showing

        # Following is used when we have to update all boxes.
        
        self.text_boxes = [self.assumps, self.goals,
                           self.add_p9, self.add_m4,
                           self.language.input]

    def update_font(self, font):
        for box in self.text_boxes:
            box.ed.SetFont(font)
            box.highlight()

        if Mac():
            # There should be a better way to do this.
            self.Show(False)
            self.Show(True)

    def reset(self):
        for box in self.text_boxes:
            box.ed.Clear()

        self.p9_options.panels.reset()
        self.m4_options.reset()

    def assemble_input(self):
        language = self.language.get_language_input().strip() + '\n'
        assumps = self.assumps.ed.GetValue().strip() + '\n'
        goals = self.goals.ed.GetValue().strip() + '\n'

        p9_add = self.add_p9.ed.GetValue().strip() + '\n'
        m4_add = self.add_m4.ed.GetValue().strip() + '\n'

        p9_triples = self.p9_options.panels.nondefaults()
        m4_triples = self.m4_options.nondefaults()

        p9_opt = option_triples_to_string(p9_triples)
        m4_opt = option_triples_to_string(m4_triples)

        input = 'set(ignore_option_dependencies). %s\n\n' % Comment_opt_dep
        if language.strip() != '':
            input += '%s\n\n%s\n' % (Comment_lang,language)
        if p9_opt.strip() != '':
            input += 'if(Prover9). %s\n%send_if.\n\n' % (Comment_p9_opt,p9_opt)
        if m4_opt.strip() != '':
            input += 'if(Mace4).   %s\n%send_if.\n\n' % (Comment_m4_opt,m4_opt)
        if p9_add.strip() != '':
            input += 'if(Prover9). %s\n%send_if.\n\n' % (Comment_p9_add,p9_add)
        if m4_add.strip() != '':
            input += 'if(Mace4).   %s\n%send_if.\n\n' % (Comment_m4_add,m4_add)

        input += '\nformulas(assumptions).\n\n%s\nend_of_list.\n\n' % assumps
        input += '\nformulas(goals).\n\n%s\nend_of_list.\n\n' % goals
        input = re.sub('\n\s*\n', '\n\n', input)  # collapse blank lines
        return input

    def store_input(self, input):

        input = utilities.remove_reg_exprs(All_added_comments, input)

        (p9,m4,assumps,goals,opt,lang,other) = partition_input.partition(input)

        # Take the part from the if(Prover9), and split those (also Mace4)

        (p9_opt, p9_other) = partition_input.extract_options(p9)
        (m4_opt, m4_other) = partition_input.extract_options(m4)

        # If there is an option set(ignore_option_dependencies), remove it
        # and tell set_options() to ignore dependencies while putting the
        # options into the GUI.

        r = re.compile('set\s*\(\s*ignore_option_dependencies\s*\)\s*\.')
        if r.match(opt):
            opt = r.sub('', opt)
            handle_dep = False
        else:
            handle_dep = True

        p9_opt_x = set_options(p9_opt, self.p9_options.panels,
                               handle_dep = handle_dep)
        if p9_opt_x != '':
            info_dialog('The following options from the if(Prover9) section'
                        ' of the input were not recognized.  They have been'
                        ' added to the "Additional Input for Prover9" box.\n\n'
                        + p9_opt_x)

        m4_opt_x = set_options(m4_opt, self.m4_options, handle_dep=handle_dep)
        if m4_opt_x != '':
            info_dialog('The following options from the if(Mace4) section'
                        ' of the input were not recognized.  They have been'
                        ' added to the "Additional Input for Mace4" box.\n\n'
                        + m4_opt_x)

        opt_x = set_options_either(opt, self.p9_options.panels,
                                   self.m4_options, handle_dep = handle_dep)

        if opt_x != '':
            info_dialog('The following options from the '
                        ' input file were not recognized.  They have been'
                        ' added to the "Additional Input for Prover9" box.\n\n'
                        + opt_x)

        self.language.input.ed.AppendText(lang.replace('.', '.\n'))
        self.assumps.ed.AppendText(assumps)
        self.goals.ed.AppendText(goals)
        self.add_p9.ed.AppendText(p9_opt_x + p9_other + opt_x + other)
        self.add_m4.ed.AppendText(m4_opt_x + m4_other)

        for box in self.text_boxes:
            box.highlight()
            box.ed.ShowPosition(0)

        self.SetSelection(1)  # Start with second page (Formulas) showing

    def store_new_input(self, input, version):
        self.reset()  # clears everything in setup tabs

        if version:
            if input.find('Saved by Prover9-Mace4') < 0:
                info_dialog('The file being opened was not created by'
                            ' Prover9-Mace4.  Therefore, it might not run'
                            ' exactly the same here as with the command-line'
                            ' versions of Prover9 or Mace4, because'
                            ' Prover9-Mace4 can rearrange the input, and'
                            ' dependent options are handled differently.')
            elif input.find('Saved by Prover9-Mace4 Version %s' % version) < 0:
                info_dialog('The file being opened was created with a'
                            ' different version of Prover9-Mace4.  This is'
                            ' usually okay, but it might cause problems if'
                            ' the underlying programs Prover9 or Mace4'
                            ' have changed.')
        self.store_input(input)

    def append_input(self, input):
        self.store_input(input)

    def on_changed(self, evt):
        sel = evt.GetSelection()
        if sel == 0:
            # Tried to redraw so that all text will appear.  Failed.
            pass

    def start_auto_highlight(self):
        for box in self.text_boxes:
            box.start_auto_highlight()

    def stop_auto_highlight(self):
        for box in self.text_boxes:
            box.stop_auto_highlight()

# END class Setup_tabs(Notebook)
                                       



#!/usr/local/bin/python3.13

#     Copyright (C) 2007 William McCune
#
#     This file is part of the LADR Deduction Library.
#
#     The LADR Deduction Library is free software; you can redistribute it
#     and/or modify it under the terms of the GNU General Public License
#     as published by the Free Software Foundation; either version 2 of the
#     License, or (at your option) any later version.
#
#     The LADR Deduction Library is distributed in the hope that it will be
#     useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with the LADR Deduction Library; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
#

import re, sys

import utilities

def in_span(i, spans):
    for (start,end) in spans:
        if i >= start and i < end:
            return True
    return False

def norm(s):
    x = s.strip()  # remove leading and trailing whitespace
    if x == '':
        return x
    else:
        return '\n' + x + '\n'

def split1(str, pat):
    r = re.compile(pat)
    comments = utilities.comment_spans(str)  # starts and ends of comments
    matched = other = ''  # matched and not_matched parts
    other_start = 0
    m = r.search(str, 0)
    while m:
        if not in_span(m.start(), comments):
            other   += str[other_start:m.start()]
            matched += str[m.start():m.end()]
            other_start = m.end()
        m = r.search(str, m.end())
    other += str[other_start:]
    return (matched, other)

def split2(str, start_pat, end_pat, remove_patterns=False):
    r1 = re.compile(start_pat)
    r2 = re.compile(end_pat)
    comments = utilities.comment_spans(str)  # starts and ends of comments
    matched = other = ''
    other_start = 0
    m1 = r1.search(str, 0)
    while m1:
        if not in_span(m1.start(), comments):
            m2 = r2.search(str, m1.end())
            while m2 and in_span(m2.start(), comments):
                m2 = r2.search(str, m2.end())
            match_end = m2.end() if m2 else len(str)
            other   += str[other_start:m1.start()]
            if remove_patterns:
                keep_start = m1.end()
                keep_end = m2.start() if m2 else len(str)
            else:
                keep_start = m1.start()
                keep_end = m2.end() if m2 else len(str)
            matched += str[keep_start:keep_end]
            next = other_start = match_end
        else:
            next = m1.end()
        m1 = r1.search(str, next)
    other += str[other_start:]
    return (matched, other)

def partition(input):

    work = input

    # if(Prover9). ... end_if.

    start_pat = 'if\s*\(\s*Prover9\s*\)\s*\.'
    end_pat   = 'end_if\s*\.'
    (p9, work) = split2(work, start_pat, end_pat, remove_patterns = True)

    # if(Prover9). ... end_if.

    start_pat = 'if\s*\(\s*Mace4\s*\)\s*\.'
    end_pat   = 'end_if\s*\.'
    (m4, work) = split2(work, start_pat, end_pat, remove_patterns = True)

    # assumptions|sos

    start_pat = 'formulas\s*\(\s*(assumptions|sos)\s*\)\s*\.'
    end_pat   = 'end_of_list\s*\.'
    (assumps, work) = split2(work, start_pat, end_pat, remove_patterns = True)

    # goals

    start_pat = 'formulas\s*\(\s*goals\s*\)\s*\.'
    end_pat   = 'end_of_list\s*\.'
    (goals, work) = split2(work, start_pat, end_pat, remove_patterns = True)

    # flags, parm, stringparms

    pat = '((set|clear)\s*\(\s*[a-z0-9_]+\s*\)\s*\.|assign\s*\(\s*[a-z0-9_]+\s*,\s*[a-z0-9_-]+\s*\)\s*\.)'
    (opt, work) = split1(work, pat)

    # op, redeclare

    pat = '(op\s*\([^,()]+,[^,()]+,[^,()]+\)\s*\.|redeclare\s*\([^,()]+,[^,()]+\)\s*\.)'
    (language, work) = split1(work, pat)

    # Clean up and return

    work = work.strip() + '\n'
    return (p9, m4, assumps, goals, opt, language, work)
                                       
# end def partition(input):

def extract_options(input):

    work = input

    # flags, parm, stringparms

    pat = '((set|clear)\s*\(\s*[a-z0-9_]+\s*\)\s*\.|assign\s*\(\s*[a-z0-9_-]+\s*,\s*[a-z0-9_]+\s*\)\s*\.)'
    (opt, work) = split1(work, pat)

    work = work.strip() + '\n'

    return (opt, work)
                                       
# end def extract_options(input):

if __name__ == '__main__':

    input = sys.stdin.read()

    (p9,m4,a,g,opt,other) = partition(input)

    print('% p9:\n' + p9)
    print('% m4:\n' + m4)
    print('% assumptions:\n' + a)
    print('% goals:\n' + g)
    print('% options:\n' + opt)
    print('% other:\n' + other)



#!/usr/local/bin/python3.13

#     Copyright (C) 2007 William McCune
#
#     This file is part of the LADR Deduction Library.
#
#     The LADR Deduction Library is free software; you can redistribute it
#     and/or modify it under the terms of the GNU General Public License
#     as published by the Free Software Foundation; either version 2 of the
#     License, or (at your option) any later version.
#
#     The LADR Deduction Library is distributed in the hope that it will be
#     useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with the LADR Deduction Library; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
#

# system imports

import os, sys
import re
import wx

# local imports

import partition_input
import utilities
from files import *
from platforms import *
from wx_utilities import *
from my_setup import *
from control import *

Program_name = 'Prover9-Mace4'
Program_version = '0.5'
Program_date = 'December 2007'

Banner = '%s Version %s, %s' % (Program_name, Program_version, Program_date)

Help = """
1. Introduction

This is a brief introduction. Better help will be available
in the next release.

Prover9-Mace4 is a front end to the programs Prover9 (which
searches for proofs) and Mace4 (which searches for finite
models and counterexamples).  Prover9 and Mace4 have been
around for a few years, and they are ordinarily run on
a command line (without a GUI).  This introduction is
mainly about the GUI.  For detailed information on Prover9
and Mace4, see http://www.cs.unm.edu/~mccune/prover9.

The GUI is intended primarily for new users and casual
users.  Power users might not be very interested, but
they might find the displays of the options useful.

Recommendations for new users who know first-order or
equational logic:

  (1) Look at some of the examples in 'File -> Sample Inputs'.
      This will give an idea of the formula syntax.

  (2) Experiment by modifying the sample inputs, both formulas
      and options.  Note:

       (a) each formula ends with a period,
       (b) variables start with (lower case) u--z,
       (c) when in doubt, include parentheses.

  (3) Start from scratch, enter some assumptions and goals,
      and try to prove or disprove some things.

Although there are many options, much can be accomplished
with just Assumptions and Goals.

2. The Main Window

The main window is divided into the Setup panel on the
left, and the Run panel on the right.

2.1. The Setup Panel

The Setup panel is for specifying the input to Prover9 and/or
Mace4.  The 'Language Options' and 'Formulas' tabs are
used for both Prover9 and Mace4.  The 'Additional Input'
tab is for some esoteric kinds of input, and it can be ignored
by most users.

2.1.1. Formulas

A few basic points about formulas (see the HTML manual for details).

  + Do not include the Prover9/Mace4 constructs
    'formulas(assumptions)' or 'end_of_list'.
  + Each formula ends with a period.
  + Variables start with (lower case) u--z.
  + The basic logic connectives are:  - | & -> <-> all exists.
  + Universal quantifiers at the outermost level can be omitted.
  + When in doubt, include parentheses.  The programs usually
    output formulas with the minimum number of parentheses,
    so you can look in the output to see how you could have
    written the formulas.

Goals

Prover9 always proves by contradiction, and Mace4 looks
for counterexamples by searching for a model of the negation
of the conjecture.  Each program puts the negation of the
goals together with the assumptions and then searches.
If there are no goals, Prover9 simply looks for a contradiction
among the assumptions, and Mace4 looks for a model of the
assumptions.  It's also acceptable to have no assumptions,
with the entire statement of the conjecture as a goal.

Multiple Goals  

If there are multiple goals, Prover9 assumes you want a
separate proof of each goal, and it will try to prove
all of them.

Mace4 searches for counterexamples to the goals.  If
there are multiple goals, Mace4 looks for structures
that falsify all of the goals.

From the HTML manual: 'If there is more than one formula in
the goals list, each must be a positive universal
conjunctive formula; that is, a formula constructed from
atomic formulas, universal quantification, and conjunction
only.'

From a logical point of view, multiple goals are a
disjunction.  Consider two goals 'P. Q.'.  If Prover9
proves either of them, it outputs a proof.  Mace4
looks for a counterexample by searching for a model
of the negation of the disjunction; that is, a model
in which both are false.

If there is any doubt about the meaning of multiple goals,
the user can combine them into a single goal with the
conjunction or disjunction operations.

2.1.2. Options Panels

The options panels contain most of the options accepted
by the programs Prover9 and Mace4, with the same names.
The GUI's default values are the same as the programs'
default values.  When an option has a non-default value,
its name is shown in red.

There can be more than one widget (button or entry box)
associated with an option.  For example, each of the Prover9
'Basic Options' also appears somewhere in 'All Options'.
When one occurrence is changed, all other occurrences are
updated.

Many of the integer-valued options specify limits and have
a range [-1, ..., some-big-number].  In these cases, -1 means
'no limit'.

Prover9 and Mace4 have a notion of 'dependent option':
changing some of the options causes others to be changed
automatically.  For example, when Prover9 receives
'set(breadth_first).', it automatically changes the parameters
'age_part', 'weight_part', 'true_part', and 'false_part'.
The GUI is aware of the dependent options.  Options
with dependencies are shown in blue, and when such an option
is changed, the dependent options are changed automatically.
Because the GUI handles dependent options, it tells
Prover9 and Mace4 to ignore option dependencies.

2.1.2.  Additional Input Panels

These can be used for Prover9 input that cannot yet be
specified with the widgets, including:

  + weighting rules,
  + KBO weights,
  + function_order and relation_order,
  + actions,
  + interpretations for semantic guidance, and
  + lists of hints (you must include 'formulas(hints).' and
    'end_of_list.').

2.2. The Run Panel

The 'Show Current Input' button at the top of the Run panel
displays the data in the Setup panel as a text file.  This
is the input that will be given to Prover9 or Mace4 when one
of the 'Start' buttons is pressed.

If you want to try for both a proof and a counterexample,
the two programs can be run concurrently (and they should
take advantage of a multiple-core processor if you have one).

The 'Info' button shows a few statistics about the search.
It can be pressed during a search or after a search has
finished.

When the program stops, the 'Show-Save' button will be enabled.
he 'Show-Save' button gives you

  (1) the input that was given to the program (not the
      'Current Input', which might have changed since
      the program was started),
  (2) the program's full output, and
  (3) if successful, proof(s) or counterexample(s).

These data will be available until the another search is started.

2.3. The Menu Bar

2.3.1.  The File Menu

'Clear Entire Setup Menu' resets everything to the initial
state: language options, formulas, options, and additional
input.

The 'Open' and 'Save' selections work as in many other
applications.  These apply to Prover9/Mace4 input files.
(The text that is 'Save'd is the same as the text displayed
by the 'Show Current Input' button in the Run panel.)
The 'Append To Input' selection is similar to the 'Open'
selection, except that existing state (formulas, options)
is not cleared first.

'Open Input File ...' should work not only on previously
'Save'd files, but also on most other Prover9 or Mace4 input
files.  Comments in 'Open'ed files are not handled well:
most comments that are not inside of formula lists appear
in 'Additional Input'.

3. Proofs and Models/Counterexamples

When a proof or model/counterexample is found, it is shown
in a separate window.  Several additional operations can
be applied to the proofs or models/counterexamples.

3.1.  Proofs

Proofs found by Prover9 are usually hard to understand
and longer then necessary.  (We are conducting research on
several methods to humanize and simplify proofs.)

For now, there is a button "Reformat...", in the Proof window,
which allows the user to transform the proof in several simple
ways.  See the HTML manual, page "Prooftrans".

3.2. Models/Counterexamples

Models/Counterexamples found by Mace4 are given as function
and relation tables.  The underlying set for domain_size=n
is always {0,...,n-1}.

There is a button "Reformat...", in the Model/Counterexample window,
which allows the user to transform the model in several simple
ways.  See the HTML manual, page "Interpformat".

3.2.1.  Isofilter.

If the user asks for more than one model (option max_models),
Mace4 will keep searching until it finds that number of models
or until it reaches some other limit.  When Mace4 finds multiple
models, many are likely to be isomorphic to others.  A separate
program 'Isofilter' can be used to remove isomorphic models.

Isofilter has several options.  One can specify the operations
that are checked for isomorphism, and one can specify the
operations that are output.  (For example, if one of the operations
is a Skolem function from an existentially quantified variable,
one might wish to omit it for checking and for output.)

One can select the the algorithm Isofilter will use:
"Occurrence Profiles" (the default) or "Canonical Forms".
They should produce the same results, but the relative
performance depends on the application.

"""

Feedback = ('\nFeedback to mccune@cs.unm.edu.  We are '
            'especially interested in making this system '
            'accessible and friendly to new users.\n')

Things_to_do = """
To Do:

 + If both Prover9 and Mace4 are running, and one
   succeeds, the user should be reminded that the
   other job should probably be killed.
 + Before quit, remind user to save unsaved input.
 + Output directly to TextCtrl, so user can see givens?
 + Comments in 'Opened' files are not handled well.
 + Remember preferences for future sessions.
 + Help

Problems:

 + Mac: On a G3 iBook (10.3.9), text boxes with a lot of text
   (>32K?) don't show scrollbars.  (All of the text really is
   there, so it can be 'Save'd.)  No problem on Intel Mac (10.4.?).
 + Mac: On the Show/Save button, the tooltip hides the selections!
 + Win32: If the Prover9 or Mace4 process is run at
   normal priority, the GUI response is poor, so Prover9
   and Mace4 are run at lower priority (Win32 only).
 + Automatic syntax highlighting causes annoying flashes
   every few seconds on Win32.  Double-buffering might help.
 + The 'Well Formed?' button sometimes highlights the wrong
   substring.  (A string can be ill-formed in one context
   and well-formed in another, and the highlighter shows
   only the first occurrence).
 
"""

class Main_frame(wx.Frame):
    """
    This is the primary Frame.
    """
    def __init__(self, parent, title, size, pos):
        wx.Frame.__init__(self, parent, -1, title, size=size, pos=pos)
        self.saved_client_size = None
        self.saved_client_pos = None
        self.current_path = None
        self.probs = {}  # for sample problems

        # self.SetBackgroundColour(wx.NamedColor('GREY50'))

        self.box_font = wx.Font(12, wx.FONTFAMILY_MODERN,
                                wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL)

        # Create the menubar
        menu_bar = wx.MenuBar()

        # File menu
        self.fmenu = wx.Menu()
        submenu = self.sample_menu(sample_dir())
        self.fmenu.AppendMenu(-1, 'Sample Inputs', submenu)
        self.fmenu.AppendSeparator()
        id = wx.NewId()
        self.fmenu.Append(id, 'Clear Entire Setup Panel')
        self.Bind(wx.EVT_MENU, self.clear_setup, id=id)
        self.fmenu.Append(wx.ID_OPEN, '&Open Input File...\tCtrl+O')
        id = wx.NewId()
        self.fmenu.Append(id, 'Append To Input...')
        self.Bind(wx.EVT_MENU, self.on_append, id=id)
        self.fmenu.AppendSeparator()
        self.fmenu.Append(wx.ID_SAVE, '&Save Input\tCtrl+S')
        self.fmenu.Enable(wx.ID_SAVE, False)
        self.fmenu.Append(wx.ID_SAVEAS, 'Save Input As...')
        self.fmenu.AppendSeparator()
        self.fmenu.Append(wx.ID_EXIT, '&Quit\tCtrl+Q')

        self.Bind(wx.EVT_MENU, self.on_open, id=wx.ID_OPEN)
        self.Bind(wx.EVT_MENU, self.on_save, id=wx.ID_SAVE)
        self.Bind(wx.EVT_MENU, self.on_saveas, id=wx.ID_SAVEAS)
        self.Bind(wx.EVT_MENU, self.on_close, id=wx.ID_EXIT)
        self.Bind(wx.EVT_CLOSE, self.on_close)
        menu_bar.Append(self.fmenu, '&File')

        # Preferences menu
        self.pref_menu = wx.Menu()

        id = wx.NewId()
        self.pref_menu.Append(id, 'Font for Text Boxes...')
        self.Bind(wx.EVT_MENU, self.select_font, id=id)
        self.pref_menu.AppendSeparator()

        self.highlight_id = wx.NewId()
        self.pref_menu.Append(self.highlight_id,
                              'Automatic Highlighting for Text Boxes', '',
                              wx.ITEM_CHECK)
        if not Win32():
            self.pref_menu.Check(self.highlight_id, True)
        self.Bind(wx.EVT_MENU, self.highlight_toggle, id=self.highlight_id)

        self.tooltip_id = wx.NewId()
        self.pref_menu.Append(self.tooltip_id, 'Show Tool Tips', '',
                              wx.ITEM_CHECK)
        self.pref_menu.Check(self.tooltip_id, True)
        self.Bind(wx.EVT_MENU, self.tooltip_toggle, id=self.tooltip_id)

        menu_bar.Append(self.pref_menu, '&Preferences')

        # View menu
        self.view_menu = wx.Menu()
        self.view_id = wx.NewId()
        self.view_menu.Append(self.view_id, 'Hide Setup Panel')
        self.Bind(wx.EVT_MENU, self.setup_toggle, id=self.view_id)
        self.run_id = wx.NewId()
        self.view_menu.Append(self.run_id,'Hide Run Panel')
        self.Bind(wx.EVT_MENU, self.run_toggle, id=self.run_id)
        menu_bar.Append(self.view_menu, '&View')

        # Help menu
        menu = wx.Menu()
        menu.Append(wx.ID_HELP_CONTENTS, 'Help \tF1')
        menu.AppendSeparator()
        menu.Append(wx.ID_ABOUT, 'About')
        self.Bind(wx.EVT_MENU, self.get_help, id=wx.ID_HELP_CONTENTS)
        self.Bind(wx.EVT_MENU, self.get_about, id=wx.ID_ABOUT)
        menu_bar.Append(menu, '&Help')

        # set MenuBar and StatusBar
        self.SetMenuBar(menu_bar)
        # self.CreateStatusBar()

        self.setup = Setup_tabs(self)
        self.control = Control_panel(self)

        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(self.setup, 1, wx.ALL|wx.GROW, 1)
        sizer.Add(self.control, 0, wx.ALL|wx.GROW, 3)
        self.SetSizer(sizer)
        # sizer.Fit(self)  # overrides size in frame init above

    def sample_menu(self, dir_path):
        " Recursive: leaves are *.in files."
        if not os.access(dir_path, os.R_OK):
            error_dialog('The samples directory %s seems to be missing' %
                         dir_path)
        else:
            menu = wx.Menu()
            entries = os.listdir(dir_path)
            entries.sort()             # sorts in-place
            have_file = have_dir = False
            for x in entries:
                if os.path.isfile(os.path.join(dir_path, x)):
                    have_file = True
            for x in entries:
                path = os.path.join(dir_path, x)
                if os.path.isdir(path):
                    submenu = self.sample_menu(path)
                    menu.AppendMenu(-1, x, submenu)
                    have_dir = True
            if have_file and have_dir:
                menu.AppendSeparator()
            for x in entries:
                path = os.path.join(dir_path, x)
                if os.path.isfile(path) and re.search('\.in$', path):
                    id = wx.NewId()
                    self.probs[id] = path
                    menu.Append(id, x)
                    self.Bind(wx.EVT_MENU, self.load_sample, id=id)
            return menu

    def load_sample(self, evt):
        path = self.probs[evt.GetId()]
        try:
            f = open(path)
            input = f.read()
            self.setup.store_new_input(input, None)
        except IOError as e:
            error_dialog('Error opening file %s for reading.' % path)

        self.SetTitle('Prover9/Mace4')

    def on_close(self, evt):
        if self.control.prover9.job_state() in [State.running,State.suspended]:
            error_dialog('You must "Kill" the Prover9 job before quitting.')
        elif self.control.mace4.job_state() in [State.running,State.suspended]:
            error_dialog('You must "Kill" the Mace4 job before quitting.')
        else:
            self.Destroy()

    def clear_setup(self, evt):
        self.setup.reset()

    def on_open(self, evt):
        (dir,style) = open_dir_style(self.current_path)  # depends on platform
        dlg = wx.FileDialog(self, message='Select a file',
                            defaultDir=dir, style=style)
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()      # full path
            try:
                f = open(path)
                input = f.read()
                self.setup.store_new_input(input, Program_version)
                self.current_path = path
                self.fmenu.Enable(wx.ID_SAVE, True)
                self.SetTitle(os.path.basename(path) + ' - Prover9/Mace4')
            except IOError as e:
                error_dialog('Error opening file %s for reading.' % path)
        dlg.Destroy()

    def on_append(self, evt):
        (dir,style) = open_dir_style(self.current_path)  # depends on platform
        dlg = wx.FileDialog(self, message='Select a file',
                            defaultDir=dir, style=style)
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()      # full path
            try:
                f = open(path)
                input = f.read()
                self.setup.append_input(input)
                # self.current_path = path
                # self.fmenu.Enable(wx.ID_SAVE, True)
                # self.SetTitle(os.path.basename(path) + ' - Prover9/Mace4')
            except IOError as e:
                error_dialog('Error opening file %s for reading.' % path)
        dlg.Destroy()

    def write_input(self, path):
        try:
            input = self.setup.assemble_input()
            f = open(path, 'w')
            f.write('%% Saved by %s.\n\n' % Banner)
            f.write(input)
            return True
        except IOError as e:
            error_dialog('Error opening file %s for writing.' % path)
            return False

    def on_save(self, evt):
        if not self.current_path:
            error_dialog('filename for save not known')
        else:
            self.write_input(self.current_path)

    def on_saveas(self, evt):
        (dir,style) = saveas_dir_style(self.current_path) # depends on platform
        dlg = wx.FileDialog(self, message='Save file as ...',
                            defaultDir=dir, style=style)
        # ??? dlg.SetFilterIndex(2)
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()      # full path
            ok = self.write_input(path)
            if ok:
                self.current_path = path
                self.fmenu.Enable(wx.ID_SAVE, True)
                self.SetTitle(os.path.basename(path) + ' - Prover9/Mace4')
            
        dlg.Destroy()

    def get_help(self, evt):
        text = '\n' + Banner + '\n' + Help
        frame = Text_frame(self, to_top(self).box_font,
                           'Help',
                           text, saveas=False)
        frame.Show(True)
        
    def get_about(self, evt):
        info_dialog('\n' + Banner + '\n' + Feedback + Things_to_do)

    def tooltip_toggle(self, evt):
        enable = self.pref_menu.IsChecked(self.tooltip_id)
        wx.ToolTip.Enable(enable)

    def highlight_toggle(self, evt):
        enable = self.pref_menu.IsChecked(self.highlight_id)
        if enable:
            self.setup.start_auto_highlight()
        else:
            self.setup.stop_auto_highlight()

    def auto_highlight(self):
        return self.pref_menu.IsChecked(self.highlight_id)

    def select_font(self, evt):
        data = wx.FontData()
        data.EnableEffects(True)
        data.SetInitialFont(self.box_font)
        dlg = wx.FontDialog(self, data)
        if dlg.ShowModal() == wx.ID_OK:
            data = dlg.GetFontData()
            font = data.GetChosenFont()
            self.box_font = font
            self.setup.update_font(font)
        dlg.Destroy()

    def setup_toggle(self, evt):
        show = (self.view_menu.GetLabel(self.view_id) == 'Show Setup Panel')
        if show:
            self.setup.Show(True)
            self.SetPosition(self.saved_client_pos)
            self.SetClientSize(self.saved_client_size)
            self.Layout()                # rearrange children
            self.view_menu.Enable(self.run_id, True)
            self.view_menu.SetLabel(self.view_id, 'Hide Setup Panel')
        else:
            # Size and position the frame (self) so that the run
            # panel stays stays the same.
            self.saved_client_pos  = (x1,y1) = self.GetPosition()
            self.saved_client_size = (w1,h1) = self.GetClientSize()
            self.setup.Show(False)
            self.Fit()  # shrink window to fit run panel
            (w2,_) = self.GetClientSize()
            self.SetPosition(((x1+w1)-w2, y1))
            self.SetClientSize((w2, h1))
            self.view_menu.Enable(self.run_id, False)
            self.view_menu.SetLabel(self.view_id, 'Show Setup Panel')
        self.Show(True)

    def run_toggle(self, evt):
        show = (self.view_menu.GetLabel(self.run_id) == 'Show Run Panel')
        if show:
            (w,h) = self.GetClientSize()
            self.control.Show(True)
            self.SetClientSize((w,h))
            self.Layout()                # rearrange children
            self.view_menu.Enable(self.view_id, True)
            self.view_menu.SetLabel(self.run_id, 'Hide Run Panel')
        else:
            # erase run panel, expand setup panel to fill space
            self.control.Show(False)
            self.setup.SetClientSize(self.GetClientSize())
            self.view_menu.Enable(self.view_id, False)
            self.view_menu.SetLabel(self.run_id, 'Show Run Panel')
        self.Show(True)

# END class Main_frame(Frame)

class Splash_screen(wx.SplashScreen):
    def __init__(self, path):

            bmp = wx.Image(path, wx.BITMAP_TYPE_GIF).ConvertToBitmap()
            wx.SplashScreen.__init__(
                self, bmp, wx.SPLASH_CENTRE_ON_SCREEN | wx.SPLASH_NO_TIMEOUT,
                5000, None, -1)

# END class Splash_screen(wx.SplashScreen):

class My_app(wx.App):
    def OnInit(self):

        size = size_that_fits((1000,700))  # reduce if screen too small
        pos = pos_for_center(size)         # position to center frame

        frame = Main_frame(None, 'Prover9/Mace4', size, pos)
        self.SetTopWindow(frame)

        if True:
            path = os.path.join(image_dir(),'prover9-splash.gif')
            if not os.access(path, os.R_OK):
                error_dialog('splash image %s not found' % path)
            else:
                splash = Splash_screen(path)
                splash.Show(True)
                time.sleep(1)
                splash.Destroy()

        frame.Show(True)
    
        # info_dialog(path_info())
    
        return True

# end class My_app(wx.App):

# Initialization that does not need wx

os.chdir(os.path.expanduser('~'))  # set current directory to user's home

app = My_app(redirect=False)

app.MainLoop()

